/*
 * Symbol.c -- Symbol table implementation
 * Written by Jeremy Nelson, 1997
 * Copyright 1997 EPIC Software Labs
 */

typedef enum {
	USER_COMMAND,
	USER_VARIABLE,
	BUILTIN_COMMAND,
	BUILTIN_VARIABLE,
	BUILTIN_FUNCTION,
	SET_VARIABLE,
	MAX_SYMBOL
} SymbolType;

typedef struct {
	char *name;
	int refcount;
	void *data[(int)MAX_SYMBOL];
} symbol;

/*
 * Used to save the datum for a symbol of a given type
 */
void set_symbol(SymbolType type, char *name, void *data);

/*
 * Used to get the datum for a symbol of a given type.
 * Really only useful when you know that you do not have
 * to deal with ambiguous symbol names, and you do not have
 * to deal with more than one symbol type.
 * Returns NULL if the symbol doesnt exist.
 */
void *get_symbol(SymbolType type, char *name, void **data);

/*
 * Used to get all of the data of a given type.
 * This is used when you need access to more than one data type,
 * and you dont need to deal with ambiguous symbol names.
 * The return value must be strictly considered a read-only value.
 * Returns NULL if the symbol doesnt exist.
 */
const void **get_symbols (char *name);

/*
 * Used to get the actual symbol table entry.  This would be used
 * when you need to deal with ambiguous symbol names.  As a side
 * effect, you also get access to all the symbols.  The return value
 * must be strictly considered a read only value.
 * Returns the first symbol that contains "name" as a leading (possibly
 * complete) subset.  Subsequent symbols would then be further possible
 * completions.  The number of such completions is put into "cnt".
 */
const symbol **get_symtab_entry (char *name, int *cnt);

